/***********************************
   Mat Bohne
   5-16-2018
   Mosquito trap timer
 ***********************************/
#include <math.h>
#include <SPI.h>
#include <Wire.h>
#include <Adafruit_GFX.h>
#include <Adafruit_SSD1306.h>
#define OLED_RESET 4
Adafruit_SSD1306 display(OLED_RESET);
int s = 0;
int m = 0;
int h = 12;
int milliDivsecs = 1000 ;
unsigned long currmillis;
boolean minlock = false;
int trigH = 12;//trigger trap on hour
int trigM = 0;// trigger trap on minute
int trigHE = 12;//trigger trap off hour
int trigME = 0;// trigger trap off minute
int bounce = 300; //button delay in milliseconds
void TRAP();//trap activation subroutine
void TIMER();//time keeping subroutine
void DISPLAYy();//display output subroutine
void SETparameters();//set current time, trap on,trap off
void diagnostics();//internal diagnostic subroutine voltage readout
void prelude();// firmware, description
void setup()
{
  pinMode(12, OUTPUT);//output to power transistor
  pinMode(11, INPUT);//input to button 1
  pinMode(10, INPUT);//input to button 2
  pinMode(2, INPUT);//jumper for fast and normal time
  pinMode(A7, INPUT);//researved
  Serial.begin(9600);
  display.begin(SSD1306_SWITCHCAPVCC, 0x3C);
  display.display();
  delay(2000);
  display.clearDisplay();
  display.setTextSize(1);
  display.setTextColor(WHITE);
  prelude();
  SETparameters();
}
void loop()
{
  TIMER();
  DISPLAYy();
  TRAP();
  if (digitalRead(11) == LOW) {
    SETparameters();
    delay(bounce);
  }
}
void TRAP() {
  if (trigH == h) {
    if (trigM == m) {
      digitalWrite(12, HIGH);
    }
  }
  if (trigHE == h) {
    if (trigME == m) {
      digitalWrite(12, LOW);
    }
  }
}
void TIMER() {
  currmillis = millis(); // currmilis== current milliseconds
  //obtain seconds from arduino time
  s = (millis() / milliDivsecs) % 60; //give seconds from miliseconds
  //next every time when secs is equal to 0 minutes are increased by 1 initial value of minutes is -1
  if ((s == 0) && (minlock == false)) // use minlock because otherwise when secs==0 mins increase more than +1
  {
    m = m + 1; // when program start and secs==0 minutes become 0
    //updateMin();//call the minutes function to update the value of minutes
    minlock = true; //block minutes increasing and increase minutes by 1
  }
  else if (s != 0)
  {
    minlock = false; //unlock minutes increasing
  }
  if (m == 60) {
    h = h + 1;
    if (h == 24) {
      h = 0;
    }
    m = 0;
  }
}
void DISPLAYy() {
  display.setCursor(0, 0);
  display.print("TIME:");
  display.print(h);
  display.print(":");
  display.print(m);
  display.print(":");
  display.print(s);
  display.setCursor(0, 8);
  display.print("trap ON ");
  display.print(trigH);
  display.print(":");
  display.print(trigM);
  display.print(":");
  display.print("00");
  display.setCursor(0, 16);
  display.print("trap OFF ");
  display.print(trigHE);
  display.print(":");
  display.print(trigME);
  display.print(":");
  display.print("00");
  diagnostics();
  display.display();
  display.clearDisplay();
}
void SETparameters() {
  display.clearDisplay();
  while (1) {
    while (1) {
      display.setCursor(0, 0);
      display.print("set current hour:");
      if (digitalRead(11) == LOW) {
        h = h + 1;
        delay(bounce);
      }
      if (h >= 24) {
        h = 0;
      }
      if (digitalRead(10) == LOW) {
        delay(bounce);
        break;

      }
      display.setCursor(0, 8);
      display.print(h);
      display.print(":");
      display.print(m);
      display.print(":");
      display.print(s);
      display.display();
      display.clearDisplay();
    }
    while (1) {
      display.setCursor(0, 0);
      display.print("set current minute:");
      if (digitalRead(11) == LOW) {
        m = m + 1;
        delay(bounce);
      }
      if (m >= 60) {
        m = 0;
      }
      if (digitalRead(10) == LOW) {
        delay(bounce);
        break;

      }
      display.setCursor(0, 8);
      display.print(h);
      display.print(":");
      display.print(m);
      display.print(":");
      display.print(s);
      display.display();
      display.clearDisplay();
    }
    while (1) {
      display.setCursor(0, 0);
      display.print("set trap ON hour:");
      if (digitalRead(11) == LOW) {
        trigH = trigH + 1;
        delay(bounce);
      }
      if (trigH >= 24) {
        trigH = 0;
      }
      if (digitalRead(10) == LOW) {
        delay(bounce);
        break;

      }
      display.setCursor(0, 8);
      display.print(trigH);
      display.print(":");
      display.print(trigM);
      display.print(":");
      display.print(s);
      display.display();
      display.clearDisplay();
    }
    while (1) {
      display.setCursor(0, 0);
      display.print("set trap ON minute:");
      if (digitalRead(11) == LOW) {
        trigM = trigM + 1;
        delay(bounce);
      }
      if (trigM >= 60) {
        trigM = 0;
      }
      if (digitalRead(10) == LOW) {
        delay(bounce);
        break;

      }
      display.setCursor(0, 8);
      display.print(trigH);
      display.print(":");
      display.print(trigM);
      display.print(":");
      display.print(s);
      display.display();
      display.clearDisplay();
    }
    while (1) {
      display.setCursor(0, 0);
      display.print("set trap OFF hour:");
      if (digitalRead(11) == LOW) {
        trigHE = trigHE + 1;
        delay(bounce);
      }
      if (trigHE >= 24) {
        trigHE = 0;
      }
      if (digitalRead(10) == LOW) {
        delay(bounce);
        break;

      }
      display.setCursor(0, 8);
      display.print(trigHE);
      display.print(":");
      display.print(trigME);
      display.print(":");
      display.print(s);
      display.display();
      display.clearDisplay();
    }
    while (1) {
      display.setCursor(0, 0);
      display.print("set trap OFF minute:");
      if (digitalRead(11) == LOW) {
        trigME = trigME + 1;
        delay(bounce);
      }
      if (trigME >= 60) {
        trigME = 0;
      }
      if (digitalRead(10) == LOW) {
        delay(bounce);
        break;
      }
      display.setCursor(0, 8);
      display.print(trigHE);
      display.print(":");
      display.print(trigME);
      display.print(":");
      display.print(s);
      display.display();
      display.clearDisplay();
    }
    while (1) {
      display.setCursor(0, 0);
      display.print("TIME ");
      display.print(h);
      display.print(":");
      display.print(m);
      display.print(":");
      display.print(s);
      display.setCursor(0, 8);
      display.print("trap ON ");
      display.print(trigH);
      display.print(":");
      display.print(trigM);
      display.print(":");
      display.print(s);
      display.setCursor(0, 16);
      display.print("trap OFF ");
      display.print(trigHE);
      display.print(":");
      display.print(trigME);
      display.print(":");
      display.print(s);
      display.setCursor(0, 24);
      display.print("parameters correct?");
      display.display();
      display.clearDisplay();
      if (digitalRead(11) == LOW) {
        delay(bounce);
        break;
      }
      if (digitalRead(10) == LOW) {
        break;

      }
    }
    if (digitalRead(10) == LOW) {
      delay(bounce);
      break;
    }
  }
}
void prelude() {
  digitalWrite(12, HIGH);
  display.setCursor(0, 0);
  display.print("Mosquito Trap");
  display.setCursor(0, 8);
  display.print("Firmware v1.2");
  display.display();
  delay(2000);
  diagnostics();
  digitalWrite(12, LOW);
  delay(2000);

}
void diagnostics() {
  int charge = readVcc();
  charge = map(charge, 3500, 4900, 1, 100);
  if (charge > 40) {
    display.setCursor(0, 24);
    display.print("Battery = ");
    display.print(charge);
    display.print("%");
    display.display();
  }
  if (charge <= 40) {
    display.setCursor(0, 24);
    display.print("SYS failure imminent ");
  }
  delay(100);
}
long readVcc() {
  // Read 1.1V reference against AVcc
  // set the reference to Vcc and the measurement to the internal 1.1V reference
#if defined(__AVR_ATmega32U4__) || defined(__AVR_ATmega1280__) || defined(__AVR_ATmega2560__)
  ADMUX = _BV(REFS0) | _BV(MUX4) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
#elif defined (__AVR_ATtiny24__) || defined(__AVR_ATtiny44__) || defined(__AVR_ATtiny84__)
  ADMUX = _BV(MUX5) | _BV(MUX0);
#elif defined (__AVR_ATtiny25__) || defined(__AVR_ATtiny45__) || defined(__AVR_ATtiny85__)
  ADMUX = _BV(MUX3) | _BV(MUX2);
#else
  ADMUX = _BV(REFS0) | _BV(MUX3) | _BV(MUX2) | _BV(MUX1);
#endif

  delay(2); // Wait for Vref to settle
  ADCSRA |= _BV(ADSC); // Start conversion
  while (bit_is_set(ADCSRA, ADSC)); // measuring

  uint8_t low  = ADCL; // must read ADCL first - it then locks ADCH
  uint8_t high = ADCH; // unlocks both

  long result = (high << 8) | low;

  result = 1125300L / result; // Calculate Vcc (in mV); 1125300 = 1.1*1023*1000
  return result; // Vcc in millivolts
}





